// $Id: views_cycle.js,v 1.2.2.12 2010/01/08 00:38:16 crell Exp $

Drupal.behaviors.viewsCycle = function(context) {

  var settings = Drupal.settings.views_cycle;

  $('.views-cycle-container:not(.viewsCycle-processed)', context).addClass('viewsCycle-processed').each(function () {

    var cycler = $(this);
    var id = cycler.attr('id');
    var config = settings[id];
    var tallest = 0;
    var verticalPadding = settings.verticalPadding || 5;
    var usingPager = false;

    // If we have a pager of some kind, create our pager placeholder.
    if (config.params.pager && config.params.pagerLocation) {
      var usingPager = true;
    }
    
    if (usingPager) {
      var pagerId = id + '-nav';
      if (config.params.pagerLocation == "after") {
        cycler.after('<ul class="view-cycle-pager" id="' + pagerId + '">');
      }
      else if (config.params.pagerLocation == "before") {
        cycler.before('<ul class="view-cycle-pager" id="' + pagerId + '">');
      }
      
      // If we're triggering the pager on mouseover, disable the transitions
      // while hovering so that the slide doesn't change out from under us.
      if (config.params.pagerEvent == 'mouseover') {
        config.params.pauseOnPagerHover = true;
      }
    }

    // Find the tallest item and set the height to that item's height + padding.
    cycler.children('li').each(function () {
      var li = $(this);
      if (li.height() > tallest) {
        tallest = li.height();
      }
    });
    config.params.height = tallest + verticalPadding;

    // Adding height to views-cycle parent div so that ie7 plays nice.
    //cycler.parents('.views-cycle').height(tallest + verticalPadding);

    // If thumbnails are used, this will always be how they're created.
    // Finds the thumbnail by the hidden div & id #.
    function makeAnchors(idx, slide) {
      // Get the cycle container and use it's id to select thumbs
      parent_selector = "#" + $(slide).parent().attr('id') + "-thumb-data";
      
      thumb_item = $(parent_selector).children().eq(idx);
      // Grab the thumbnail item's content and class names.
      return Drupal.theme('viewsCycle', thumb_item.html(), thumb_item.attr('class'), idx);
    }

    // We can't set the function from PHP because we can't specify a datatype of
    // "function" from PHP.  So we simply flag it to use the function above.
    if (config.use_pager_callback) {
      config.params.pagerAnchorBuilder = makeAnchors;

      // jQuery.cycle is supposed to add an activeSlide class to the currently
      // active thumbnail.  However, for unknown reasons it doesn't seem to be
      // doing so.  We work around it by adding our own class after the main
      // animation runs.  It's not as slick because it doesn't happen in parallel,
      // but it's close enough until that bug gets fixed.
      config.params.after = function (currEle, nextEle, opts, flag) {
        $(currEle)
          .parents('.views-cycle')
          .find('.view-cycle-pager > li')
          .filter('.view-cycle-thumb-' + opts.currSlide).addClass('activeSlide').end()
          .not('.view-cycle-thumb-' + opts.currSlide).removeClass('activeSlide').end();
      };
    }

    // Fire away!
    cycler.cycle(config.params);

    // jquery.cycle breaks links within the pager.  I'm not entirely sure how,
    // but the event itself never fires.  However, we can then re-attach our 
    // own event to simulate the link being clicked.  It's weird, but it works.
    if (usingPager && config.params.pagerEvent == 'mouseover') {
      $("#" + pagerId + ' a').click(function() {
        window.location = this.href;
      });
    }
  });
};

Drupal.theme.prototype.viewsCycle = function(body, classes, id) {
  return '<li class="' + classes + '">' + body + "</li>\n";
}
